unit uService;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, WinSvc;

const
  SERVICE_WIN32_OWN_PROCESS=$00000010;
  SERVICE_DEMAND_START=$00000003;
  SERVICE_ERROR_NORMAL=$00000001;
  EVENTLOG_ERROR_TYPE=$0001;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

procedure DoService(AServiceName, AServiceDisplayName:string);

var
  Form1: TForm1;

implementation

{$R *.DFM}

var
  FServiceName: string;
  FServiceDisplayName: string;
  FServiceStatusHandle:SERVICE_STATUS_HANDLE;
  FServiceStatus:TServiceStatus;
  FErrorCode:DWORD;
  FServiceTableEntry:array [0..1] of TServiceTableEntry;

function GetLastErrorText:string;
var
  Size:DWORD;
  Temp:LPSTR;
begin
  Size := 512;
  Temp := nil;
  try
    GetMem(Temp, Size);
    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM or FORMAT_MESSAGE_ARGUMENT_ARRAY,
    nil, GetLastError, LANG_NEUTRAL, Temp, Size,nil);
  finally
    Result := StrPas(Temp);
    FreeMem(Temp);
  end;
end;

procedure AddToMessageLog(Msg:string);
var
  a:array [0..1] of string;
  hEventSource:THandle;
begin
  hEventSource := RegisterEventSource(nil, PCHAR(FServiceName));
  if hEventSource > 0 then begin
    a[0] := FServiceName + ' error: ' + IntToStr(FErrorCode);
    a[1] := Msg;
    ReportEvent(hEventSource, EVENTLOG_ERROR_TYPE, 0, 0, nil, 2, 0, @a, nil);
    DeregisterEventSource(hEventSource);
  end;
end;

function ReportStatusToSCMgr(State, ExitCode, Wait:DWORD):BOOL;
begin
  with FServiceStatus do begin
    if (State = SERVICE_START_PENDING) then
      dwControlsAccepted := 0
    else
      dwControlsAccepted := SERVICE_ACCEPT_STOP;
    dwCurrentState := State;
    dwWin32ExitCode := ExitCode;
    dwWaitHint := Wait;
    if (State = SERVICE_RUNNING) or (State = SERVICE_STOPPED) then
      dwCheckPoint:=0
    else
      inc(dwCheckPoint);
  end;
  Result := SetServiceStatus(FServiceStatusHandle, FServiceStatus);
  if not Result then AddToMessageLog('SetServiceStauts');
end;

procedure ServiceStop;
begin
  Application.Terminate;
  Application.ProcessMessages;
end;

procedure ServiceStart;
begin
  if not ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) then exit;
  Application.Initialize;
  Application.ShowMainForm := False;
  Application.CreateForm(TForm1, Form1);
  if not ReportStatusToSCMgr(SERVICE_RUNNING, NO_ERROR, 0) then exit;
  Application.Run;
end;

procedure Handler(CtrlCode:DWORD); stdcall;
begin
  case CtrlCode of
  SERVICE_CONTROL_STOP:
    begin
      ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 0);
      ServiceStop;
      ReportStatusToSCMgr(SERVICE_STOPPED, GetLastError, 0);
      exit;
    end;
  end;
  ReportStatusToSCMgr(FServiceStatus.dwCurrentState, NO_ERROR, 0);
end;

procedure ServiceMain;
begin
  FServiceStatusHandle := RegisterServiceCtrlHandler(PCHAR(FServiceName), ThandlerFunction(@Handler));
  if FServiceStatusHandle = 0 then begin
    ReportStatusToSCMgr(SERVICE_STOPPED, GetLastError, 0);
    exit;
  end;
  FServiceStatus.dwServiceType := SERVICE_WIN32_OWN_PROCESS;
  FServiceStatus.dwServiceSpecificExitCode := 0;
  FServiceStatus.dwCheckPoint := 1;
  if not ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) then begin
    ReportStatusToSCMgr(SERVICE_STOPPED, GetLastError, 0);
    exit;
  end;
  ServiceStart;
end;

procedure InstallService;
var
  ServiceControlHandle: SC_HANDLE;
  SCManagerHandle: SC_HANDLE;
  Path: LPSTR;
  Size: DWORD;
begin
  Size := 512;
  GetMem(Path, Size);
  if GetModuleFileName(0, Path, Size) = 0 then begin
    FreeMem(Path);
    WriteLn('Unable to install ' + FServiceName + ', GetModuleFileName Fail.');
    exit;
  end;
  FreeMem(Path);

  SCManagerHandle := OpenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
  if (SCManagerHandle > 0) then begin
    ServiceControlHandle := CreateService(SCManagerHandle ,PCHAR(FServiceName), PCHAR(FServiceDisplayName),
                            SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS, SERVICE_DEMAND_START,
                            SERVICE_ERROR_NORMAL, Path, nil, nil, nil, nil, nil);
    if (ServiceControlHandle > 0) then begin
      WriteLn('Install Ok.');
      CloseServiceHandle(ServiceControlHandle);
    end else
      WriteLn('Unable to install ' + FServiceName + ', CreateService Fail.');
  end else
    WriteLn('Unable to install ' + FServiceName + ', OpenSCManager Fail.');
end;

procedure UnInstallService;
var
  ServiceControlHandle: SC_HANDLE;
  SCManagerHandle: SC_HANDLE;
begin
  SCManagerHandle := OpenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
  if (SCManagerHandle > 0) then begin
    ServiceControlHandle := OpenService(SCManagerHandle, PCHAR(FServiceName), SERVICE_ALL_ACCESS);
    if (ServiceControlHandle > 0) then begin
      if ControlService(ServiceControlHandle, SERVICE_CONTROL_STOP, FServiceStatus) then begin
        Write('Stopping Service ');
        Sleep(1000);
        while (QueryServiceStatus(ServiceControlHandle, FServiceStatus)) do begin
          if FServiceStatus.dwCurrentState = SERVICE_STOP_PENDING then begin
            Write('.');
            Sleep(1000);
          end else
            break;
        end;
        WriteLn;

        if FServiceStatus.dwCurrentState = SERVICE_STOPPED then
          WriteLn('Service Stop Now')
        else begin
          CloseServiceHandle(ServiceControlHandle);
          CloseServiceHandle(SCManagerHandle);
          WriteLn('Service Stop Fail');
          exit;
        end;
      end;
      if (DeleteService(ServiceControlHandle)) then
        WriteLn('Service Uninstall Ok.')
      else
        WriteLn('DeleteService fail (' + GetLastErrorText + ').');
      CloseServiceHandle(ServiceControlHandle);
    end else
      WriteLn('OpenService fail (' + GetLastErrorText + ').');
    CloseServiceHandle(SCManagerHandle);
  end else
    WriteLn('OpenSCManager fail (' + GetLastErrorText + ').');
end;

procedure DoService(AServiceName, AServiceDisplayName:string);
begin
  FServiceName := AServiceName;
  FServiceDisplayName := AServiceDisplayName;
  if (ParamCount=1) then begin
    if ParamStr(1)='/?' then begin
      WriteLn('----------------------------------------');
      WriteLn(Uppercase(FServiceName) + ' usage help');
      WriteLn('----------------------------------------');
      WriteLn(Uppercase(FServiceName) + ' /install to install the service');
      WriteLn(Uppercase(FServiceName) + ' /remove to uninstall the service');
      WriteLn(Uppercase(FServiceName) + ' /? Help');
      Halt;
    end else if Uppercase(copy(ParamStr(1),1 ,2)) = '/I' then begin
      InstallService;
      Halt;
    end else if Uppercase(copy(ParamStr(1), 1, 2))='/R' then begin
      UnInstallService;
      Halt;
    end else begin
      WriteLn('Unknown parameter: ' + ParamStr(1));
      Halt;
    end;
  end;

  // Setup service table which define all services in this process
  with FServiceTableEntry[0] do begin
    lpServiceName := PCHAR(FServiceName);
    lpServiceProc := TServiceMainFunction(@ServiceMain);
  end;

  // Last entry in the table must have nil values to designate the end of the table
  with FServiceTableEntry[1] do begin
    lpServiceName := nil;
    lpServiceProc := nil;
  end;
  if not StartServiceCtrlDispatcher(FServiceTableEntry[0]) then begin
    AddToMessageLog('StartServiceCtrlDispatcher Error!');
    Halt;
  end;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  beep;
end;

end.
